/* 
 * File:   gdgelement.h
 * Author: Michal Hucik <hucik@ordoz.com>
 *
 * Created on 13. května 2019, 22:56
 * 
 * 
 * ----------------------------- License -------------------------------------
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * ---------------------------------------------------------------------------
 */


#ifndef GDGELEMENT_H
#define GDGELEMENT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <glib.h>
#include "vhdl_optimized_cb.h"


    typedef enum en_GDGELEMENT_PORT_TYPE {
        GDGELEMENT_PORT_IN = 0,
        GDGELEMENT_PORT_OUT,
        GDGELEMENT_PORT_INOUT
    } en_GDGELEMENT_PORT_TYPE;


    typedef struct st_GDGELEMENT_PORT {
        char *name;
        char *spice_netlist_pin_name;
        int id;
        en_GDGELEMENT_PORT_TYPE type;
    } st_GDGELEMENT_PORT;


    typedef struct st_GDGELEMENT {
        char *name;
        char *description;
        int count_port;
        int count_outputs;
        st_GDGELEMENT_PORT **port;
        gboolean have_optimized_code;
        char *logic_operator;
        vhdl_optimized_cb vhdl_cb;
        gboolean is_buffer;
    } st_GDGELEMENT;


    typedef enum en_GDGELEMENTSALL_TYPE {
        GDGELEMENTSALL_TYPE_MASTER = 0,
        GDGELEMENTSALL_TYPE_SLAVE,
    } en_GDGELEMENTSALL_TYPE;


    typedef struct st_GDGELEMENTSALL {
        en_GDGELEMENTSALL_TYPE type;
        int count_element;
        st_GDGELEMENT **element;
        int *count_instances;
    } st_GDGELEMENTSALL;


    extern st_GDGELEMENTSALL* gdgelements_init ( gboolean verbose );
    extern st_GDGELEMENTSALL* gdgelements_create_slave_from_elements ( st_GDGELEMENT **elements, int elements_count );
    extern void gdgelements_destroy ( st_GDGELEMENTSALL *elmall );

    extern st_GDGELEMENT* gdgelements_search_element_by_name ( st_GDGELEMENTSALL *elmall, const char *name );
    extern void gdgelement_register_part_instance ( st_GDGELEMENTSALL *elmall, st_GDGELEMENT *elm, void *part );
    extern void gdgelement_unregister_part_instance ( st_GDGELEMENTSALL *elmall, st_GDGELEMENT *elm, void *part );
    extern st_GDGELEMENT_PORT* gdgelement_get_port_by_spice_netlist_pin_name ( st_GDGELEMENT *elm, char *spice_netlist_pin_name );
    extern st_GDGELEMENT_PORT* gdgelement_get_port_by_name ( st_GDGELEMENT *elm, char *port_name );

#ifdef __cplusplus
}
#endif

#endif /* GDGELEMENT_H */

