/* 
 * File:   gdgpart.h
 * Author: Michal Hucik <hucik@ordoz.com>
 *
 * Created on 13. května 2019, 22:38
 * 
 * 
 * ----------------------------- License -------------------------------------
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * ---------------------------------------------------------------------------
 */


#ifndef GDGPART_H
#define GDGPART_H

#ifdef __cplusplus
extern "C" {
#endif

#include <glib.h>
#include "gdgnet.h"

#define GDGPART_PRINT_NAME_PREFIX "x"


    typedef struct st_GDGPART {
        st_GDGELEMENT *element;
        int loc_x;
        int loc_y;
        st_GDGNET **netpin;
    } st_GDGPART;


    typedef enum en_GDGPARTSALL_TYPE {
        GDGPARTSALL_TYPE_MASTER = 0,
        GDGPARTSALL_TYPE_SLAVE,
    } en_GDGPARTSALL_TYPE;


    typedef struct st_GDGPARTSALL {
        en_GDGPARTSALL_TYPE type;
        st_GDGELEMENTSALL *elmall;
        int count_part;
        st_GDGPART **part;
        int min_loc_x;
        int min_loc_y;
        int max_loc_x;
        int max_loc_y;
        int xsize;
        int ysize;
        st_GDGPART **loc;
    } st_GDGPARTSALL;

    extern st_GDGPARTSALL* gdgparts_new ( st_GDGELEMENTSALL *elmall, en_GDGPARTSALL_TYPE type );
    extern st_GDGPARTSALL* gdgparts_init ( st_GDGELEMENTSALL *elmall, const char *filepath );
    extern st_GDGPARTSALL* gdgparts_create_slave_from_parts ( st_GDGPART **parts, int count_parts, st_GDGELEMENTSALL *elmall );
    extern void gdgparts_destroy ( st_GDGPARTSALL *partall );
    extern void gdgparts_remove_part ( st_GDGELEMENTSALL *elmall, st_GDGPARTSALL *partall, st_GDGPART *remove_part );

    extern st_GDGPART* gdgparts_get_part_by_location ( st_GDGPARTSALL *partall, int loc_x, int loc_y, gboolean print_error );
    extern char* gdgpart_create_name ( st_GDGPART *part );
    extern void gdgparts_replace_net ( st_GDGPARTSALL *partall, st_GDGNET *oldnet, st_GDGNET *newnet );

    extern gboolean gdgparts_reindex_part_locations ( st_GDGPARTSALL *partall );
    extern void gdgparts_add_part ( st_GDGPARTSALL *partall, st_GDGELEMENT *elm, int loc_x, int loc_y );

#ifdef __cplusplus
}
#endif

#endif /* GDGPART_H */

